function mda1D
%*************************************************************************
%*   MDA1D: Morphospace-Disparity Analysis for one dimentional data      *
%*   MDA1D (C)2001 Nicolas Navarro                                       *
%* ***********************************************************************
global filename         % Default name of input file (scores on principal components)
global Eigenfile        % Default name of input file (eigenvalues for standardization of PCs)
global Occurrencefile                % Default name of input file (temporal or geographical...occurrences)
global Groupfile                    % Default name of input file (group member)

if not(exist('IntrMDA.mat'))
   [IntrImage, IntrMap] = imread( 'imagMDAvs1.2.bmp' );
   save IntrMDA IntrImage IntrMap;
   delete('imagMDAvs1.2.bmp');
end;
load( 'IntrMDA.mat' );
[NbL, NbC, NbP] = size( IntrImage );

IntroFigure = figure( ...
		'Visible', 'off', ...
		'NumberTitle', 'off', ...
		'Units', 'pixels', ...
	   'Position',[300 500 475 200], ...
		'Color', [0.8 0.8 0.8], ...
		'Colormap', IntrMap, ...
		'KeyPressFcn', 'close(''gcf'')', ...
		'WindowStyle', 'modal' );
set( gca,'Position', [0 0 1 1] ); 
IntroImage = image( IntrImage );  axis off; axis image;
set( IntroImage, 'ButtonDownFcn', 'close(''gcf'')' );
set( IntroFigure, 'Visible', 'on' );

% wait user action
uiwait(IntroFigure);
clear all;
init=1;

while init==1
stp=not(exist('stop.mat'));
if stp~=1
   delete('stop.mat');
   break;
end;

% MDA version and copyright
MDA_version;
fprintf(' this is a one-dimensional data version\n restricted to 10 variables and to SGA/BTailTest functions\n\n');
pause;

% loading default values of various parameters of analysis
MDA_prefs;

%Text user interface for Morphospace-Disparity Analysis
if not(exist(filename))
   [filename, pathname]=uigetfile('*.txt','Load TXT file:');
   if filename~=0
      filename=[pathname filename];
      PCs=load (filename);
      if isempty(PCs)
         string=['File' filename ' is empty!!!.'];
         title='File Control Failed';
         errordlg(string,title,'modal');
      end; 
   end;
else
   PCs= load (filename);
end;

[lig coll]=size(PCs);

%Option1: Rescaling PCs
disp('   PCs need rescaling PCs to eigenvalues: y/n');
disp('   Help:');
disp('   If you use data from some statistical package, all variances on axes can have be  ');
disp('   scale to unity. In this case the rescaling option (using the root squared of the  ');
disp('   eigenvalue)is used to obtain variance on PCs equal to their eigenvalue and so equal ');
fprintf('   to their initial proportion of variance\n\n');

Analysis_Charact.Standardization = input('   Rescaling on eigenvalue: ','s');                
if Analysis_Charact.Standardization == 'y'
   if not(exist(Eigenfile))
      [Eigenfile, pathname]=uigetfile('*.txt','Load TXT file: eigenvalues for rescaling');
      if filename~=0
            Eigenfile=[pathname Eigenfile];
            eigenvalues=load (Eigenfile);
         if isempty(eigenvalues)
            string=['File' Eigenfile ' is empty!!!.'];
            title='File Control Failed';
            errordlg(string,title,'modal');
         end; 
      end;
   else
      eigenvalues= load (Eigenfile);
   end;
   for yy=1:coll
      PCs(:,yy) = PCs(:,yy).*sqrt(eigenvalues(:,yy));
   end;
elseif Analysis_Charact.Standardization == 'n'
   PCs=PCs;
else
   PCs=PCs;
   warning('no standardization by eigenvalues is used');
end;
[l p] = size(PCs);

%Option2: Reduce number of PCs
Analysis_Charact.PCs_retain = input('\n   number of variables selected: ');
if Analysis_Charact.PCs_retain>p
   Analysis_Charact.PCs_retain=p;
   fprintf('\n   !!! warning choice of variables retained upper that number of variables present\n '); 
   fprintf('  in the initial data matrix; analysis performs on this number of variables present:\n '); 
   fprintf('  ==>> Number of max available variables = %i\t',p); 
end;     
PCs = PCs(:,1:Analysis_Charact.PCs_retain); %New data matrix
if p>1
   fprintf('\n   !!! warning choice this function is for one dimensional data\n '); 
   fprintf('  each column are treated separately\n '); 
end;     

%Option3: Choice of one type of analysis
fprintf('\n   Four analyses are available:\n');
fprintf('\n   (1) SGA: single group analysis');
fprintf('\n   (2) BTailTest: Bootstrap Tail Test\n\n');

analysis_perform = input('   choice: ');
if analysis_perform==1
   Analysis_Charact.analysis_perform='SGA';
   SGA(PCs,Analysis_Charact);
elseif analysis_perform==2
   Analysis_Charact.analysis_perform='BTailTest';
   BTailTest(PCs,Analysis_Charact);
end;
end;
%-----------------------------------------------------
function SGA(PCs,Analysis_Charact);
[sp npc]=size(PCs);
global Occurrencefile                % Default name of input file (temporal or geographical...occurrences)

%loading binary matrix of occurrence (temporal, geographical...occurrence) 
if not(exist(Occurrencefile))
   [filename, pathname]=uigetfile('*.txt','Load TXT file: Temporal, geographical...occurrence');
   if filename~=0
      filename=[pathname filename];
      occurrence=load (filename);
      if isempty(occurrence)
         string=['File' filename ' is empty!!!.'];
         title='File Control Failed';
         errordlg(string,title,'modal');
      end; 
   end;
else
   occurrence=load(Occurrencefile);
end;
[ind interv]=size(occurrence);
if ind~=sp
   error('    number of observation in occurrence file different that this in data_matrix file');
end;
min_obs=min(sum(occurrence));

%Option3: Rarefaction for some quantifiers
[Analysis_Charact.rmenu,samplesize_used]=Rarefaction_Menu(min_obs);

%Option4: Bootstrap menu Nb_Bootstrap_resampling
Analysis_Charact.Nb_Bootstrap_resampling = input('\n   Number of bootstrap resamplings: ');

%Perform Disparity analysis
res=main_analysis(PCs,occurrence,1,samplesize_used,interv,Analysis_Charact);

%Option5: Bootstrap estimator [mean standard deviation Confidence Interval (upper value lower value)]
CL = input('\n   Do you want a confidence interval (e.g. 95%) on the bootstrap y/n\n   case n: error bar = +/- 1 std\n   choice: ','s');
if CL=='n'
   Analysis_Charact.CI='+/- 1 stdev';
elseif CL=='y'
   CL = input('\n   input level of Confidence interval (e.g., 95): ');
   CI=sprintf('%i',CL);
   Analysis_Charact.CI=strcat(CI,'%');
end;
for i=1:5
   [res.Estim(i).mean,res.Estim(i).std,res.Estim(i).lowval,res.Estim(i).uppval,res.Estim(i).lowrang,res.Estim(i).upprang]=bootstat(res.Estim(i).Bootsampl,CL);
end;

%Plot Menu (limit to 10 variables)
Tmin=0.5;
Tmax=interv+0.5;
if npc>10
   nsubplt=10;
else
   nsubplt=npc;
end;
for i=1:5
    res.Estim(i).mean=permute(res.Estim(i).mean,[3,2,1]);
    res.Estim(i).std=permute(res.Estim(i).std,[3,2,1]);
    res.Estim(i).lowval=permute(res.Estim(i).lowval,[3,2,1]);
    res.Estim(i).uppval=permute(res.Estim(i).uppval,[3,2,1]);
    res.Estim(i).lowrang=permute(res.Estim(i).lowrang,[3,2,1]);
    res.Estim(i).upprang=permute(res.Estim(i).upprang,[3,2,1]);
end;
%graph: disparity vs time
%graph: Pattern of occupation vs time
figMean=figure( ...
   'Name','Mean(max of 10 variables): mean and +/- one std of bootstrap sample',...
        'Units','centimeters', ...
        'Position',[10 1 15 18]);
for nsplt=1:nsubplt
    strg2=sprintf('Mean on Variable %i',nsplt);
    Mmax=max(res.Estim(1).uppval(:,nsplt))+ceil(max(res.Estim(1).std(:,nsplt))./2);
    Mmin=min(res.Estim(1).lowval(:,nsplt))-ceil(max(res.Estim(1).std(:,nsplt))./2);
    Unit=Mmin+(Mmax-Mmin)./12;
    plotting(res.Estim(1).mean(:,nsplt),res.Estim(1).lowrang(:,nsplt),res.Estim(1).upprang(:,nsplt),strg2,nsplt,Tmin,Tmax,Mmin,Mmax,Unit);
end;
figMinMax=figure( ...
   'Name','Rarefied Min & Max(max of 10 variables): mean and +/- one std of bootstrap sample',...
        'Units','centimeters', ...
        'Position',[10 1 15 18]);
for nsplt=1:nsubplt
    strg2=sprintf('Min & Max on PC%i',nsplt);
    MMmax=max(res.Estim(3).uppval(:,nsplt))+ceil(max(res.Estim(3).std(:,nsplt))./2);
    MMmin=min(res.Estim(2).lowval(:,nsplt))-ceil(max(res.Estim(2).std(:,nsplt))./2);
    Unit=MMmin+(MMmax-MMmin)./12;
    plotting(res.Estim(2).mean(:,nsplt),res.Estim(2).lowrang(:,nsplt),res.Estim(2).upprang(:,nsplt),strg2,nsplt,Tmin,Tmax,MMmin,MMmax,Unit);
    plotting(res.Estim(3).mean(:,nsplt),res.Estim(3).lowrang(:,nsplt),res.Estim(3).upprang(:,nsplt),strg2,nsplt,Tmin,Tmax,MMmin,MMmax,Unit);
end;
figRange=figure( ...
        'Name','Range (max of 10 variables): mean and +/- one std of bootstrap sample', ...
        'Units','centimeters', ...
        'Position',[0.2 1 15 18]);
strg=char('Univariates ranges');
Rmin=0;
for nsplt=1:nsubplt
    strg2=sprintf('Range of variable %i',nsplt);
    Rmax=max(res.Estim(4).uppval(:,nsplt))+ceil(max(res.Estim(4).std(:,nsplt))./2);
    Unit=Rmax./12;
    plotting(res.Estim(4).mean(:,nsplt),res.Estim(4).lowrang(:,nsplt),res.Estim(4).upprang(:,nsplt),strg2,nsplt,Tmin,Tmax,Rmin,Rmax,Unit);
end;
figVar=figure( ...
        'Name','Variance(max of 10 variables): mean and +/- one std of bootstrap sample', ...
        'Units','centimeters', ...
        'Position',[0.2 1 15 18]);
strg=char('Univariates variances');
Vmin=0;
for nsplt=1:nsubplt
    strg2=sprintf('Variance of variable %i',nsplt);
    Vmax=max(res.Estim(5).uppval(:,nsplt))+ceil(max(res.Estim(5).std(:,nsplt))./2);
    Unit=Vmax./12;
    plotting(res.Estim(5).mean(:,nsplt),res.Estim(5).lowrang(:,nsplt),res.Estim(5).upprang(:,nsplt),strg2,nsplt,Tmin,Tmax,Vmin,Vmax,Unit);
end;

%Save Menu
fprintf('\n\n   ===================================\n');
Save_choice = input('   Save numerical results?\n   -> data are appended if file name already exists\n   (y/n): ','s');
if Save_choice=='y'
   Save_function(Analysis_Charact,1,interv,res);
elseif Save_choice=='n'
   repeat_choice = input('   Start again analysis(y) or exit (x)?: ','s');
   if repeat_choice=='x'
      stop=1;
      save stop
   end;
end;

%-----------------------------------------------------
function BTailTest(PCs,Analysis_Charact);
[sp npc]=size(PCs);
global Groupfile                    % Default name of input file (group member)

fprintf('\n   Help Bootstrap Tail Test:\n');
fprintf('\n   >> The first group in group file is the one use for comparison');
fprintf('\n   This group is resampling with replacement (bootstrap)[except for one metric see below]');
fprintf('\n   and observed value of other groups are compared with this obtained distribution.');
fprintf('\n   Because disparity estimates can be increase or decrease, three possibilities are given:');
fprintf('\n     - one-tailed test with the two cases: upper tail or lower tail');
fprintf('\n     - two-tailed test.');
fprintf('\n\n   Range-based disparity estimates, sensitive to sample size, are not available for this analysis. ');
fprintf('\n\n   1000 bootstrap resampling is a minimum value for a good accuracy of results.\n');

%loading binary matrix of group member(taxinomic, geographic...groups) 
if not(exist(Groupfile))
   [filename, pathname]=uigetfile('*.txt','Load TXT file: group file with first column is the reference group');
   if filename~=0
      filename=[pathname filename];
      group=load (filename);
      if isempty(group)
         string=['File' filename ' is empty!!!.'];
         title='File Control Failed';
         errordlg(string,title,'modal');
      end; 
   end;
else
   group=load(Groupfile);
end;
[ind ngrps]=size(group);
if ind~=sp
   error('    number of observation in occurrence file different that this in data_matrix file');
end;

%Option: Choice of one disparity estimate => reduce time of analysis, permits more bootstrap replications
fprintf('\n   Just one Disparity estimate available:');
fprintf('\n     (1) Variance on each axis');
DispEstimate= 1;
Analysis_Charact.DispEstimate=char(sprintf('Sum of variances'));

%Option3: Rarefaction for some quantifiers
Analysis_Charact.rmenu='none';

%Option4: Bootstrap menu Nb_Bootstrap_resampling
Analysis_Charact.Nb_Bootstrap_resampling = input('\n\n   Number of bootstrap resamplings: ');
if Analysis_Charact.Nb_Bootstrap_resampling<1000
   disp('   number of resamplings is low: results may be inaccurate');
end;

%Option5: No statististic on bootstrap sample
Analysis_Charact.CI='none';

%Perform Disparity Analysis on first group
index_group = find(group(:,1));
Gdata = PCs(index_group,:);
[lig col]=size(Gdata);
step=1;
mstep=Analysis_Charact.Nb_Bootstrap_resampling./10;
bound=0:mstep:Analysis_Charact.Nb_Bootstrap_resampling;
bound(1)=1;
for r=1:Analysis_Charact.Nb_Bootstrap_resampling
   if r==bound(step)
      fprintf('\n   >>> bootstrap iteration n%i',bound(step));
      fprintf(' on %i',Analysis_Charact.Nb_Bootstrap_resampling);
      step=step+1;
   end;
      bootsampl=resamplk(Gdata,lig);                         %bootstrap sample 
      %Dispersion estimate
      res.Bootsampl(r,:)=var(bootsampl);                     %univariate variance
end;

%Observed values for disparity estimates for for group 1 to N 
for ng=1:ngrps
      index_group = find(group(:,ng));
      ii=length(index_group);
      Gdata = PCs(index_group,:);
      res.analysis_parameters(ng,1)=ng;                                              %index of groups with 1 = group used for comparison 
      %Dispersion estimate
         fprintf('\n   >>> studied group n%i',ng);
         fprintf(' on %i',ngrps);
      res.obs_value(ng,:)=var(Gdata);                                             %Sum of univariate variance (or Total Variance) (e.g. Wills et 1994, Foote XXX, Eble 2000 Paleobiology)
end;

%find probabilities for each descriptor for group 2 to N-1 (suppress group 1 => comparison)
for ng=1:ngrps
   M=mean(res.Bootsampl(:,:));
   for nc=1:col
      index_valueBT = find(abs(res.Bootsampl(:,nc)-M(1,nc)) >= abs(res.obs_value(ng,nc)-M(1,nc)));
      numbersBT = length(index_valueBT);
      res.p_twotail(ng,nc) = numbersBT./Analysis_Charact.Nb_Bootstrap_resampling;
      index_valueUTS = find(res.Bootsampl(:,nc) >= res.obs_value(ng,nc));
      numbersUTS = length(index_valueUTS);
      res.p_upperonetail(ng,nc) = numbersUTS./Analysis_Charact.Nb_Bootstrap_resampling;
      index_valueUTI = find(res.Bootsampl(:,nc) <= res.obs_value(ng,nc));
      numbersUTI = length(index_valueUTI);
      res.p_loweronetail(ng,nc) = numbersUTI./Analysis_Charact.Nb_Bootstrap_resampling;
      
      %re-initialize
      index_valueBT=[]; 
      numbersBT=[];
      index_valueUTS=[];
      numbersUTS=[];
      index_valueUTI=[];
      numbersUTI=[];
   end;      
   M=[];
end;

%Plot Menu
%graph: Histogram of bootsample of disparity estimate
min_val=min(res.Bootsampl);
min_obs=min(res.obs_value);
if min_obs<min_val
   min_val=min_obs;
end;
max_val=max(res.Bootsampl);
max_obs=max(res.obs_value);
if max_obs>max_val
   max_val=max_obs;
end;
nplot=Analysis_Charact.PCs_retain;
if nplot>10
   nplot=10;
else
   nplot=nplot;
end;

strg=strcat(Analysis_Charact.DispEstimate,sprintf(' (Bootstrap resamplings = %i',Analysis_Charact.Nb_Bootstrap_resampling),...
   sprintf(')'));
figDisp=figure( ...
   'Name','Disparity', ...
        'Units','pixels', ...
        'Position',[28 120 450 600]);
Charact= uicontrol('Parent',figDisp, ...
	'Units','centimeters', ...
	'FontName','Arial', ...
	'FontSize',10, ...
	'FontWeight','bold', ...
	'HorizontalAlignment','left', ...
	'ListboxTop',0, ...
	'Position',[0.2 0.2 10 0.5], ...
	'String',strg, ...
	'Style','text', ...
	'Tag','charact');

for nsplot=1:nplot
        subplot(5,2,nsplot);
        strg2=sprintf('Variable %i', nsplot);
        hist(res.Bootsampl(:,nsplot),30);
        xlabel(strg2,'FontName','Geneva','FontSize',9,'FontWeight','bold');
end;
     
%Save Menu
fprintf('\n\n   ===================================\n');
Save_choice = input('   Save numerical results?\n   -> data are appended if file name already exists\n   (y/n): ','s');
if Save_choice=='y'
   Save_function(Analysis_Charact,ngrps,1,res);
elseif Save_choice=='n'
   repeat_choice = input('   Start again analysis(y) or exit (x)?: ','s');
   if repeat_choice=='x'
      stop=1;
      save stop
   end;
end;

%--------------------------------------------------------
function res=main_analysis(data,Occur,groupindex,samplesize_used,interv,Analysis_Charact);
res.analysis_parameters=cat(2,zeros(interv,3),groupindex*ones(interv,1));
for j=1:interv  
   fprintf('\n   >>> studied sample n%i',j);
   fprintf(' on %i',interv);
   indoccur = find(Occur(:,j));
   subdata = data(indoccur,:);
   [lig col]=size(subdata);
   if lig>1
   if samplesize_used=='obs'
      Rsamplesize=lig;
   elseif samplesize_used>lig
      Rsamplesize=lig;
      fprintf('\n   for one sample, rarefaction size is upper to sample size:\n  analysis is performed with sample size !!\n ');   
   else
      Rsamplesize=samplesize_used;
   end;
   for r=1:Analysis_Charact.Nb_Bootstrap_resampling
         bootsampl=resamplk(subdata,lig);             %bootstrap sample 
      if samplesize_used~=lig
         bootsampk=resamplk(subdata,Rsamplesize);     %bootstrap sample rarefied to k specimens
      else
         bootsampk=bootsampl;
      end;
      %Location estimated
      res.Estim(1).Bootsampl(r,:,j)=mean(bootsampl);                        %mean value of N-onedimensional data
      res.Estim(2).Bootsampl(r,:,j)=min(bootsampk);                        %Rarefied minimum value of N-onedimensional data 
      res.Estim(3).Bootsampl(r,:,j)=max(bootsampk);                        %Rarefied maximum value of N-onedimensional data
      %Dispersion estimate
      res.Estim(4).Bootsampl(r,:,j)=unirange(bootsampk);                %N univariate range 
      res.Estim(5).Bootsampl(r,:,j)=var(bootsampl);                     %N univariate variance 
      %re-initialize
      bootsampl=[];
      bootsampk=[];
   end;
   res.analysis_parameters(j,1)=lig;             %Sample size
   res.analysis_parameters(j,2)=Rsamplesize;     %Rarefied sample size
   res.analysis_parameters(j,3)=j;               %e.g. temporal subdivision (depends on nature of data contain in occurrence file)
   
   %re-initialize for the next temporal subdivision 
   Rsamplesize=samplesize_used;
   %lacking data
   elseif lig<1
      for i=1:5
         res.Estim(i).Bootsampl(1,:,j)=nan*ones(1,col);
      end;
   elseif lig==1
      for i=1:5
         res.Estim(i).Bootsampl(1,:,j)=subdata;
      end;
   end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-----------------------------------------------------
% subfunctions for bootstrap resampling 
%-----------------------------------------------------
function Dboot=resamplk(data,k)
[l p]=size(data);
Index_random=Intrndunif(l,k,1);
Dboot=data(Index_random,:); %bootstrap resampling of k specimens from l

%------------------------------------------------------
function [out1,out2]=Gresampl(data,grp,k)
%used for Partial disparity analysis
[l p]=size(data);
Index_random=Intrndunif(l,k,1);
out1=data(Index_random,:);
out2=grp(Index_random,:);

%------------------------------------------------------
function indrandom=Intrndunif(a,b,c)
indunif=rand(b,c);
rescalind=a.*indunif;
indrandom=ceil(rescalind);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------------------------------
% subfunctions for disparity estimates
%------------------------------------------------------
function variance=var(X)
StandDev=std(X);
variance=StandDev.^2;

%------------------------------------------------------
function unrang=unirange(X)
Minima=min(X);
Maxima=max(X);
unrang=Maxima-Minima;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------------------------------
% subfunctions for bootstrap statistic (mean std CL)
%------------------------------------------------------
function [Mean,stdev,lowval,uppval,LowRang,UppRang]=bootstat(data,CL)
[l p]=size(data);
Mean=mean(data);
stdev=std(data);
if CL=='n'
   uppval=Mean+stdev;
   lowval=Mean-stdev;
   UppRang=stdev;
   LowRang=stdev;
else
   [lowval, uppval]=ConfInterv(data, CL);
   UppRang=uppval-Mean;
   LowRang=Mean-lowval;
end;
%------------------------------------------------------
function [Lower, Upper]=ConfInterv(data, CL)
%Confidence interval on bootstrap estimator based on 
%percentile method
[re ax int]=size(data);
val=(((100-CL)./100)./2);
q1=ceil(re*val);
q2=ceil(re-re*val);
for ii=1:ax
   for j=1:int
       ranked=sort(data(:,ii,j));
       Upper(1,ii,j)=ranked(q2);
       Lower(1,ii,j)=ranked(q1);
   end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------   
%Plotting functions
%-------------------------------------------------------
function plotting(Yvalue,lower,upper,leg,nsplot,Xmin,Xmax,Ymin,Ymax,Unit)
X=1:length(Yvalue);
subplot(5,2,nsplot);
sbp=errorbar(X,Yvalue,lower,upper,'k'); 
set(sbp,'LineWidth',1);
axis([Xmin Xmax Ymin Ymax])
hold on;
text(Xmin+0.05,Unit,leg);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------------------------------
%Rarefaction Menu
%------------------------------------------------------
function [rmenu,samplesize_used]=Rarefaction_Menu(min_obs)
fprintf('\n   Rarefaction Menu:\n');
fprintf('\n     Help:');
fprintf('\n     Some estimate of morphospace occupation are sensitive to sample size:');
fprintf('\n\t-Total range,\n\t-N Root of Product of ranges,\n\t-Range,\n\t-Area of convex hull,\n\t-Min & Max on PC  ');
fprintf('\n     Variance-based estimates are not sensitive to sample size and their analyses');
fprintf('\n     are not based on rarefied sample');
fprintf('\n     The rarefaction procedure is used to eliminated this link (Foote 1992 Paleobiology)');
fprintf('\n     The rarefaction is performed using bootstrapping (Eble 2000 Paleobiology).\n');
fprintf('\n   (1) no rarefaction');
fprintf('\n   (2) rarefaction with minimum observed sample size = %i\t',min_obs);
fprintf('\n   (3) rarefaction with a user-choice sample size\n');
rarefaction = input('\n   choice:  ');
if rarefaction == 1
   samplesize_used = 'obs';
   rmenu=sprintf('no rarefaction: size used: obs');
elseif rarefaction == 2
   samplesize_used = min_obs;  
   rmenu=sprintf('y: rarefaction size used: %i',min_obs);
elseif rarefaction == 3
   samplesize_used = input('\n   sample size of rarefaction: ');  
   rmenu=sprintf('y: rarefaction size used: %i',samplesize_used);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------
%Save as:
%-------------------------------------------------------
function Save_function(Analysis_Charact,ngroup,interv,res);
%erase files if necessary
[file, path] = uiputfile( '.txt', 'Save results as:');
outfile=fullfile(path, file);
%Analysis Characteristic
Summary_string=strcat(sprintf('\n\n   Analysis Summary: MDA one-dimensional'),...
   sprintf('\n   ==================================='),...
   sprintf('\n   Analysis performed: %s',Analysis_Charact.analysis_perform),...
   sprintf('\n   Rarefaction: %s',Analysis_Charact.rmenu),...
   sprintf('\n   Number of bootstrap resamplings: %i',Analysis_Charact.Nb_Bootstrap_resampling),...
   sprintf('\n   Upper-lower values: %s',Analysis_Charact.CI));
switch Analysis_Charact.analysis_perform
   case {'BTailTest'}
      Summary_string=strcat(Summary_string,sprintf('\n   Disparity metric used: %s',Analysis_Charact.DispEstimate));
end;
%Print Analysis Characteristics
disp(Summary_string);
Summary_outfile=strcat(Summary_string,sprintf('\n\n')); 
fid = fopen(outfile,'a');
fprintf(fid,Summary_outfile,'uint8');
fclose(fid);
switch Analysis_Charact.analysis_perform
   case {'SGA'}
   %format data
   string_outfile=strcat(sprintf('\n\nSample Size'),...
      sprintf('\tRarefaction Size'),...
      sprintf('\tInterval'),...
      sprintf('\tGroup'));

   col=Analysis_Charact.PCs_retain;
   if col>10
      col=10;%limit Min Max values to 10 first PCs
   end;
   for Naxis=1:col
      Mean_string=sprintf('Mean%i',Naxis);
      Mean_string=strcat(sprintf('\t%s',Mean_string),sprintf('\tstdev%s',Mean_string),sprintf('\tupper value%s',Mean_string),sprintf('\tlower value%s',Mean_string));
      Min_string=sprintf('Min%i',Naxis);
      Min_string=strcat(sprintf('\t%s',Min_string),sprintf('\tstdev%s',Min_string),sprintf('\tupper value%s',Min_string),sprintf('\tlower value%s',Min_string));
      Max_string=sprintf('Max%i',Naxis);
      Max_string=strcat(sprintf('\t%s',Max_string),sprintf('\tstdev%s',Max_string),sprintf('\tupper value%s',Max_string),sprintf('\tlower value%s',Max_string));
      Rang_string=sprintf('Rang%i',Naxis);
      Rang_string=strcat(sprintf('\t%s',Rang_string),sprintf('\tstdev%s',Rang_string),sprintf('\tupper value%s',Rang_string),sprintf('\tlower value%s',Rang_string));
      Var_string=sprintf('Var%i',Naxis);
      Var_string=strcat(sprintf('\t%s',Var_string),sprintf('\tstdev%s',Var_string),sprintf('\tupper value%s',Var_string),sprintf('\tlower value%s',Var_string));
	   string_outfile=strcat(string_outfile,Mean_string,Min_string,Max_string,Rang_string,Var_string);
   end

   for G=1:ngroup
      for Naxis=1:col
         for i=1:5
             temp(i).mm=cat(2,res(G).Estim(i).mean(:,Naxis),res(G).Estim(i).std(:,Naxis),res(G).Estim(i).uppval(:,Naxis),res(G).Estim(i).lowval(:,Naxis));
         end;
      temp(Naxis).Location=cat(2,temp.mm);
      end;
      tmp(G).Loc=cat(2,temp.Location);
   end;
   Location=cat(1,tmp.Loc);
   [uu vv]=size(Location);
   res_analysis_parameters=cat(1,res.analysis_parameters);
   for nint=1:interv*ngroup
      string_res=sprintf('\n%i\t%i\t%i\t%i',res_analysis_parameters(nint,:));
      string_outfile=strcat(string_outfile,string_res);
      temp_MinMax=Location(nint,:);
      for i=1:vv
         string_outfile=strcat(string_outfile,sprintf('\t%f',temp_MinMax(i)));
      end;
   end;
   string_outfile=strcat(string_outfile,sprintf('\n\n')); 
        
   fid = fopen(outfile,'a');
   fprintf(fid,string_outfile,'uint8');
   fclose(fid);
   
   repeat_choice = input('\n   Start again analysis(y) or exit (x)?: ','s');
   if repeat_choice=='x'
      stop=1;
      save stop
   end;
   
case {'BTailTest'}
   col=Analysis_Charact.PCs_retain;
   if col>10
      col=10;%limit Min Max values to 10 first PCs
   end;
%format data
   string_outfile=strcat(sprintf('\n\nGroup'));
   for Naxis=1:col
      Obsvalue_string=sprintf('Variab %i',Naxis);
      Obsvalue_string=strcat(sprintf('\t%s',Obsvalue_string),sprintf('\tp two-tail%s',Obsvalue_string),...
         sprintf('\tp upper one-tail%s',Obsvalue_string),sprintf('\tp lower one-tail%s',Obsvalue_string));
   	string_outfile=strcat(string_outfile,Obsvalue_string);
  end
   
   string_outfile=strcat(string_outfile,sprintf('\n'));
   for nc=1:col
      all_result(:,:,nc)=cat(2,res.obs_value(:,nc),res.p_twotail(:,nc),res.p_upperonetail(:,nc),res.p_loweronetail(:,nc));
   end;
   [uu vv hh]=size(all_result);
   all_result=reshape(all_result,[uu,vv*hh]);
   [uuu vvv]=size(all_result);
   for ng=1:ngroup
      string_res=sprintf('\n%i',res.analysis_parameters(ng,1));
      string_outfile=strcat(string_outfile,string_res);
      all_result_temp=all_result(ng,:);
      for i=1:vvv
         string_outfile=strcat(string_outfile,sprintf('\t%f',all_result_temp(i)));
      end;  
   end;
   string_outfile=strcat(string_outfile,sprintf('\n\n')); 
   fprintf('\n   -----------------------------------\n');   
   disp(string_outfile);
   fid = fopen(outfile,'a');
   fprintf(fid,string_outfile,'uint8');
   fclose(fid);
   
   repeat_choice = input('\n   Start again analysis(y) or exit (x)?: ','s');
   if repeat_choice=='x'
      stop=1;
      save stop
   end;
end;

%*******************************************************
