%===============================================
%function for preservation and time averaging
%===============================================
%Can be used on the results of randclade.m function
%Steps (0.2 myr) are grouped in 14 bins representing 5 myr. 
%Data set is altered to simulated incomplete, time-averaged fossil record with constant probability of preservation.
%Preserved occurences of species are regrouped within bins. The morphological mean of a species in preserved samples
%is considered as the species' morphology. At the end, a correction for Lazarus taxa is incorporated in occurrence file.
%Input for preservation rates are based on Raup and Foote (1996, Paleobiology) results.
%
function res=pres_clade(clade,SSclade,clrand)

[species dim time] = size(clade);
[l hh]=size(SSclade);
%rate of preservation per interval (200.000 yrs) 
%=> 0.25 per 0.7 myrs (=0.35 Lma)[Foote and Raup 1996 for Cenozoic mammals]
p_pres=0.07;
%Bin of 5 myrs (lower limit) convert into steps.
Lbin=[1	26	51	76	101	126	151	176	201	226	251	276	301	326];  
range=Lbin(2)-Lbin(1);
nbins=length(Lbin);

occurence=1-isnan(clade(:,1,:));
occurence=reshape(occurence,species,time);

for jj=1:time
  for ii=1:species
     if occurence(ii,jj)==1
        preservation=rand;
        if preservation<p_pres
           occurence(ii,jj)=1;
        else
           occurence(ii,jj)=0;
           clade(ii,:,jj)=ones(1,2)*nan;
        end;
     end;
  end;
end;

for nb=1:1:nbins
   Llimit=Lbin(nb);
   if nb==nbins
      ULimit=Lbin(nb)+range;
   else
      ULimit=Lbin(nb+1)-1;
   end;
   indsteps=Llimit:ULimit;
   occbin=occurence(:,indsteps);
   for ii=1:species
      if sum(occbin(ii,:))>=1
         occur(ii,nb)=1;
      for j=1:dim
         mm=clade(ii,j,:);
         mmm=mm(find(1-isnan(mm)));
         if isempty(mmm)
            c=nan;
         else
            c=mean(mmm);
         end;
         MeanM(ii,j)=c;
      end;
      else
         occur(ii,nb)=0;
      end;
   end;      
end;
somm=sum(occur,2);
indexsauv=find(somm~=0);
yy=length(indexsauv);
res.morpho=MeanM(indexsauv,:);
clear MeanM;

res.occurrence=occur(indexsauv,:);
clear occur;
res.occur_LAZARcorr=lazarT(res.occurrence);
res.LazarT=sum(res.occur_LAZARcorr-res.occurrence);

group=zeros(species,hh);
for h=1:hh
   index=SSclade(h).species;
   kk=length(index);
   for sp=1:species
      for jj=1:kk
      if sp==index(jj)
         group(sp,h)=1;
      end;
      end;
   end;
end;
res.group_all=group(indexsauv,:);
clselect=cat(1,1,clrand);
res.group_select=res.group_all(:,clselect');

%-----------------------------------------
%subroutine for Lazarus taxa
%-----------------------------------------
function cc=lazarT(occurrence)
[sp time]=size(occurrence);
cc=occurrence;
for ii=1:sp
   kk=find(occurrence(ii,:));
   if length(kk)>2
      cc(ii,kk(1):kk(end))=ones(1,kk(end)-kk(1)+1);
   end;
end;
   


